import os
import random
import subprocess
import tempfile
import time
from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
import platform
from datetime import date
from multiprocessing import Lock
from multiprocessing.managers import SyncManager

from rotate.utils import fopen

in_progress_tag = '.in-progress'

def new_filename(default_output, pps, ext, gzip=False, bzip2=False, in_progress=False):
    hostname = platform.node()
    dirname, basename = os.path.split(default_output)
    if dirname:
        os.makedirs(dirname, exist_ok=True)
    if basename:
        basename += '.'
    timestamp = int(time.time())
    dt = date.fromtimestamp(timestamp)
    datestr = dt.strftime('%Y%m%d')
    filename = os.path.join(dirname, '{base}{host}.{date}.{time}.{pps}.{ext}'.format(
        base=basename, host=hostname, date=datestr, time=timestamp, pps=pps, ext=ext))
    if gzip:
        filename += '.gz'
    elif bzip2:
        filename += '.bz2'
    if in_progress:
        filename += in_progress_tag
    return filename

def cmd_scamper(pps, tmp, write, ftype, sccmd):
    cmd = 'scamper -O {ftype} -p {pps} -c "{sccmd}" -f {infile} {write}'.format(ftype=ftype, pps=pps, sccmd=sccmd, infile=tmp, write=write)
    return cmd

def main():
    parser = ArgumentParser(formatter_class=ArgumentDefaultsHelpFormatter)
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument('-f', '--input')
    group.add_argument('-i', '--addr', nargs='*')
    parser.add_argument('-p', '--pps', default=5000, type=int, help='Packets per second.')
    parser.add_argument('-o', '--default-output', required=True)
    group = parser.add_mutually_exclusive_group()
    group.add_argument('-z', '--gzip', action='store_true')
    group.add_argument('-b', '--bzip2', action='store_true')
    parser.add_argument('--remote')
    parser.add_argument('--cycles', type=int, default=1)
    parser.add_argument('--random', action='store_true')
    parser.add_argument('-O', '--extension', choices=['warts', 'json'], default='json')
    parser.add_argument('--in-progress', action='store_true', help='Mark file in progress with .in-progress')
    parser.add_argument('--address', default='127.0.0.1:50000')
    parser.add_argument('--debug', type=int)
    args, remaining = parser.parse_known_args()

    host, _, port = args.address.partition(':')
    port = int(port)
    lock = Lock()
    infile = args.input
    SyncManager.register('get_lock', callable=lambda: lock)
    SyncManager.register('get_infile', callable=lambda: infile)
    manager = SyncManager((host, port), authkey=b'test')
    manager.start()

    try:
        sccmd = ' '.join(remaining)

        cycle = 0
        while args.cycles == 0 or cycle < args.cycles:
            f = tempfile.NamedTemporaryFile(mode='wt', delete=False)
            tmp = f.name
            try:
                if args.input:
                    with lock:
                        with fopen(args.input, 'rt') as g:
                            for line in g:
                                if args.random:
                                    addr, _, _ = line.rpartition('.')
                                    addr = '{}.{}'.format(addr, random.randint(0, 255))
                                else:
                                    addr = line.strip()
                                f.write('{}\n'.format(addr))
                else:
                    f.writelines('{}\n'.format(addr) for addr in args.addr)
                f.close()

                ftype = args.extension
                filename = new_filename(args.default_output, args.pps, ftype, gzip=args.gzip, bzip2=args.bzip2, in_progress=args.in_progress)
                if args.gzip:
                    write = '| gzip > {}'.format(filename)
                elif args.bzip2:
                    write = '| bzip2 > {}'.format(filename)
                else:
                    write = '-o {}'.format(filename)
                cmd = cmd_scamper(args.pps, tmp, write, ftype, sccmd)
                print(cmd)
                if args.debug:
                    with fopen(filename, 'wt') as f:
                        f.write(cmd + '\n')
                    time.sleep(args.debug)
                else:
                    start = time.time()
                    subprocess.run(cmd, shell=True, check=False)
                    end = time.time()
                    secs = end - start
                    mins = secs / 60
                    hours = mins / 60
                    print('Duration: {:,.2f} s {:,.2f} m {:,.2f} h'.format(secs, mins, hours))
                if args.in_progress:
                    os.rename(filename, filename[:-len(in_progress_tag)])
                try:
                    cycle += 1
                except OverflowError:
                    cycle = 1
            finally:
                os.unlink(tmp)
    finally:
        manager.shutdown()



import os
import random
import subprocess
import tempfile
import time
from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter

from rotate.utils import fopen, new_filename

def cmd_trace(proto, pps, first_hop, tmp, write, ftype):
    if proto == 'icmp':
        proto = 'icmp-paris'
    elif proto == 'udp':
        proto = 'udp-paris'
    elif proto == 'tcp':
        proto = 'tcp'
    else:
        raise Exception('Unknown proto {}'.format(proto))
    cmd = 'sudo scamper -O {type} -p {pps} -c "trace -P {proto} -f {first}" -f {infile} {write}'.format(type=ftype, pps=pps, proto=proto, first=first_hop, infile=tmp, write=write)
    return cmd

def cmd_ping(proto, pps, tmp, write, ftype, options):
    if proto == 'icmp':
        proto = 'icmp-echo'
    elif proto == 'udp':
        proto = 'udp'
    elif proto == 'tcp':
        proto = 'tcp-syn'
    else:
        raise Exception('Unknown proto {}'.format(proto))
    if options is None:
        options = ''
    cmd = 'sudo scamper -O {type} -p {pps} -c "ping -P {proto}{options}" -f {infile} {write}'.format(type=ftype, pps=pps, proto=proto, options=options, infile=tmp, write=write)
    return cmd

def main():
    parser = ArgumentParser(formatter_class=ArgumentDefaultsHelpFormatter)
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument('-i', '--input')
    group.add_argument('-I', '--addr', nargs='*')
    parser.add_argument('-p', '--pps', default=8000, type=int, help='Packets per second.')
    parser.add_argument('-P', '--proto', default='icmp', choices=['icmp', 'udp', 'tcp'], help='Transport protocol.')
    parser.add_argument('-d', '--default-output', required=True)
    group = parser.add_mutually_exclusive_group()
    group.add_argument('-z', '--gzip', action='store_true')
    group.add_argument('-b', '--bzip2', action='store_true')
    parser.add_argument('-r', '--remote')
    parser.add_argument('--cycles', type=int, default=1)
    parser.add_argument('-R', '--random', action='store_true')
    parser.add_argument('-e', '--extension', choices=['warts', 'jsonl'], default='jsonl')
    sp = parser.add_subparsers(required=True)
    trace = sp.add_parser('trace')
    trace.add_argument('-f', '--first-hop', type=int, default=1)
    trace.set_defaults(method='trace')
    ping = sp.add_parser('ping')
    ping.add_argument('-c', '--count', type=int, default=4)
    ping.add_argument('-o', '--replycount', type=int)
    ping.set_defaults(method='ping')
    args = parser.parse_args()

    cycle = 0
    while args.cycles == 0 or cycle < args.cycles:
        f = tempfile.NamedTemporaryFile(mode='wt', delete=False)
        tmp = f.name
        try:
            if args.input:
                with fopen(args.input, 'rt') as g:
                    for line in g:
                        if args.random:
                            addr, _, _ = line.rpartition('.')
                            addr = '{}.{}'.format(addr, random.randint(0, 255))
                        else:
                            addr = line.strip()
                        f.write('{}\n'.format(addr))
            else:
                f.writelines('{}\n'.format(addr) for addr in args.addr)
            f.close()

            if args.extension == 'warts':
                ftype = 'warts'
            else:
                ftype = 'json'
            filename = new_filename(args.default_output, args.proto, args.pps, args.extension, gzip=args.gzip, bzip2=args.bzip2)
            if args.gzip:
                write = '| gzip > {}'.format(filename)
            elif args.bzip2:
                write = '| bzip2 > {}'.format(filename)
            else:
                write = '-o {}'.format(filename)
            dirname, basename = os.path.split(args.default_output)
            pattern = os.path.join(dirname, '{}.warts*'.format(basename))
            if args.method == 'trace':
                cmd = cmd_trace(args.proto, args.pps, args.first_hop, tmp, write, ftype)
            elif args.method == 'ping':
                cmd = cmd_ping(args.proto, args.pps, tmp, write, ftype, args.options)
            else:
                raise Exception('Unknown method: {}'.format(args.method))
            print(cmd)
            start = time.time()
            subprocess.run(cmd, shell=True, check=False)
            end = time.time()
            secs = end - start
            mins = secs / 60
            hours = mins / 60
            print('Duration: {:,.2f} s {:,.2f} m {:,.2f} h'.format(secs, mins, hours))
            try:
                cycle += 1
            except OverflowError:
                cycle = 1
        finally:
            os.unlink(tmp)



import os
from argparse import ArgumentParser
from multiprocessing.managers import SyncManager

from rotate.utils import fopen

def main():
    parser = ArgumentParser()
    parser.add_argument('-a', '--address', default='127.0.0.1:50000')
    parser.add_argument('-f', '--filename')
    args = parser.parse_args()

    host, _, port = args.address.partition(':')
    port = int(port)
    SyncManager.register('get_lock')
    SyncManager.register('get_infile')
    manager = SyncManager((host, port), authkey=b'test')
    manager.connect()
    lock = manager.get_lock()

    outfile = str(manager.get_infile())[1:-1]
    print(outfile)

    lock.acquire()
    os.rename(outfile, '{}.bak'.format(outfile))
    with fopen(outfile, 'wt') as f, fopen(args.filename, 'rt') as g:
        for line in g:
            f.write(line)
    lock.release()



import os
import random
from argparse import ArgumentParser
from multiprocessing import Pool
import pandas as pd

from cloudtrace.trace.utils import fopen

_infile = None

def random_addr(addr):
    addr, _, _ = addr.rpartition('.')
    addr = '{}.{}'.format(addr, random.randint(0, 255))
    return addr

def random_dests(outfile, infile=None):
    if infile is None:
        infile = _infile
    with fopen(infile, 'rt') as f, fopen(outfile, 'wt') as g:
        for line in f:
            addr = random_addr(line)
            g.write('{}\n'.format(addr))

def random_order(outfile, infile=None):
    if infile is None:
        infile = _infile
    targets = []
    with fopen(infile, 'rt') as f:
        for line in f:
            addr = random_addr(line)
            targets.append(addr)
    random.shuffle(targets)
    with fopen(outfile, 'wt') as g:
        for addr in targets:
            g.write('{}\n'.format(addr))

def main():
    global _infile
    parser = ArgumentParser()
    parser.add_argument('-f', '--infile', required=True)
    parser.add_argument('-r', '--random-order', action='store_true')
    subparsers = parser.add_subparsers()
    single = subparsers.add_parser('single')
    single.add_argument('-o', '--outfile', required=True)
    excel = subparsers.add_parser('excel')
    excel.add_argument('-i', '--instances', required=True)
    excel.add_argument('-s', '--sheet', required=True)
    excel.add_argument('-p', '--processes', type=int, default=30)
    excel.add_argument('-d', '--dir', required=True)
    group = excel.add_mutually_exclusive_group()
    group.add_argument('-z', '--gzip', action='store_true')
    group.add_argument('-b', '--bzip2', action='store_true')

    args = parser.parse_args()
    _infile = args.infile
    if args.instances:
        df = pd.read_excel(args.instances, sheet_name=args.sheet)
        outfiles = []
        for row in df.itertuples():
            basename = '{}.targets'.format(row.Name)
            if args.gzip:
                basename += '.gz'
            elif args.bzip2:
                basename += '.bz2'
            outfile = os.path.join(args.dir, basename)
            outfiles.append(outfile)
        with Pool(args.processes) as pool:
            func = random_order if args.random_order else random_dests
            for _ in pool.imap_unordered(func, outfiles):
                pass
    else:
        if args.random_order:
            random_order(args.outfile, args.infile)
        else:
            random_dests(args.outfile, args.infile)



from setuptools import setup, find_packages

setup(
    name="rotate-scamper",
    version='0.0.3',
    author='Alex Marder',
    # author_email='notlisted',
    description="Create prefix-to-AS mappings.",
    url="https://github.com/alexmarder/traceutils",
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'rotate-scamper=rotate.rotate_scamper:main',
            'rotate-targets=rotate.rotate_targets:main'
        ],
    },
    zip_safe=False,
    include_package_data=True,
    python_requires='>3.7'
)



import bz2
import gzip
import os
import platform
import time
from datetime import date

def fopen(filename, mode='rt', *args, **kwargs):
    if filename.endswith('.gz'):
        return gzip.open(filename, mode, *args, **kwargs)
    elif filename.endswith('.bz2'):
        return bz2.open(filename, mode, *args, **kwargs)
    return open(filename, mode, *args, **kwargs)

def new_filename(default_output, proto, pps, ext, gzip=False, bzip2=False):
    hostname = platform.node()
    dirname, basename = os.path.split(default_output)
    if dirname:
        os.makedirs(dirname, exist_ok=True)
    if basename:
        basename += '.'
    timestamp = int(time.time())
    dt = date.fromtimestamp(timestamp)
    datestr = dt.strftime('%Y%m%d')
    filename = os.path.join(dirname, '{base}{host}.{date}.{time}.{proto}.{pps}.{ext}'.format(
        base=basename, host=hostname, date=datestr, time=timestamp, proto=proto, pps=pps, ext=ext))
    if gzip:
        filename += '.gz'
    elif bzip2:
        filename += '.bz2'
    return filename